#include <Adafruit_SSD1306.h>

#define switched true
#define triggered true
#define interrupt_trigger_type RISING
#define debounce 10

#define intensity 500

const int PHOTO1_PIN = A0;
const int PHOTO2_PIN = A1;
const int PHOTO3_PIN = A2;
const int PHOTO4_PIN = A3;

const int LED1 = 13;
const int LED2 = 12;
const int LED3 = 11;
const int LED4 = 10;

const int BUZZER_PIN = 9;
const int BUTTON_PIN = 3;

int type_tone = 1;

int NOTE1;
int NOTE2;
int NOTE3;
int NOTE4;

const int BUZZER_OFF = 0;

int cnt = 0;

bool initialisation_complete = false;
volatile bool interrupt_process_status = !triggered;

Adafruit_SSD1306 display(128, 32, &Wire);

void  button_interrupt_handler()
{
  if (initialisation_complete == true) {
    if (interrupt_process_status == !triggered) {
      if (digitalRead(BUTTON_PIN) == HIGH) {
        interrupt_process_status = triggered;
      }
    }
  }
}


bool read_button()
{
  int button_reading;

  static bool switching_pending = false;
  static long int elapse_timer;

  if (interrupt_process_status == triggered) {
    button_reading = digitalRead(BUTTON_PIN);

    if (button_reading == HIGH)  {
      switching_pending = true;
      elapse_timer = millis();
    }

    if (switching_pending && button_reading == LOW) {
      if (millis() - elapse_timer >= debounce) {
        switching_pending = false;
        interrupt_process_status = !triggered;
        return switched;
      }
    }
  }

  return !switched;
}

void press_button()
{
  
  cnt++;

  if(cnt == 2){
    cnt = 0;
    type_tone = 0;
  } else {
    type_tone = 1;
  }
  
  
}

void setup() {
  pinMode(BUZZER_PIN, OUTPUT);
  noTone(BUZZER_PIN);
  pinMode(BUTTON_PIN, INPUT);

  pinMode(LED1, OUTPUT);
  pinMode(LED2, OUTPUT);
  pinMode(LED3, OUTPUT);
  pinMode(LED4, OUTPUT);

  Serial.begin(9600);
  
  attachInterrupt(digitalPinToInterrupt(BUTTON_PIN),
                  button_interrupt_handler,
                  interrupt_trigger_type);

  initialisation_complete = true;
  
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.clearDisplay();
  display.setTextColor(WHITE);
  display.setTextSize(1);
  int textWidth = 6 * strlen("Vrei sa canti?");
  int x = (display.width() - textWidth) / 2;
  int y = (display.height() - 8) / 2;
  display.setCursor(x, y);
  display.print("Vrei sa canti?");
  display.display();
  delay(100);
}

void loop() {
  if (read_button() == switched) {
    display.clearDisplay();
  display.setCursor(0, 0);
  int textWidth = 6 * strlen("Tonalitate schimbata");
  int x = (display.width() - textWidth) / 2;
  int y = (display.height() - 8) / 2;
  display.setCursor(x, y);
  display.print("Tonalitate schimbata");
  display.display();
  delay(500);
    press_button();

  }
  
  int photo1 = analogRead(PHOTO1_PIN);
  int photo2 = analogRead(PHOTO2_PIN);
  int photo3 = analogRead(PHOTO3_PIN);
  int photo4 = analogRead(PHOTO4_PIN);


  if (photo1 >= intensity && photo2 >= intensity && photo3 >= intensity &&
      photo4 >= intensity) {
    digitalWrite(BUZZER_PIN, BUZZER_OFF);
    display.clearDisplay();
    int textWidth = 6 * strlen("Vrei sa canti?");
    int x = (display.width() - textWidth) / 2;
    int y = (display.height() - 8) / 2;
    display.setCursor(x, y);
    display.print("Vrei sa canti?");
    display.display();
  }

  if (type_tone == 1) {
    NOTE1 = 165;
    NOTE2 = 196;
    NOTE3 = 220;
    NOTE4 = 247;
  } else {
    NOTE1 = 330;
    NOTE2 = 392;
    NOTE3 = 440;
    NOTE4 = 494;
  }
  if (photo1 < intensity) {
    display.clearDisplay();
    int textWidth = 6 * strlen("NOTE 1");
    int x = (display.width() - textWidth) / 2;
    int y = (display.height() - 8) / 2;
    display.setCursor(x, y);
    display.print("NOTE 1");
    if (type_tone == 0) {
      display.print("   *");
    }
    display.display();
    analogWrite(LED1, 255);
    playNote(NOTE1);
  } else if (photo2 < intensity) {
    display.clearDisplay();
    int textWidth = 6 * strlen("NOTE 2");
    int x = (display.width() - textWidth) / 2;
    int y = (display.height() - 8) / 2;
    display.setCursor(x, y);
    display.print("NOTE 2");
    if (type_tone == 0) {
      display.print("   *");
    }
    display.display();
    analogWrite(LED2, 255);
    playNote(NOTE2);
  } else if (photo3 < intensity) {
    display.clearDisplay();
    int textWidth = 6 * strlen("NOTE 3");
    int x = (display.width() - textWidth) / 2;
    int y = (display.height() - 8) / 2;
    display.setCursor(x, y);
    display.print("NOTE 3");
    if (type_tone == 0) {
      display.print("   *");
    }
    display.display();
    analogWrite(LED3, 255);
    playNote(NOTE3);
  } else if (photo4 < intensity) {
    display.clearDisplay();
    int textWidth = 6 * strlen("NOTE 4");
    int x = (display.width() - textWidth) / 2;
    int y = (display.height() - 8) / 2;
    display.setCursor(x, y);
    display.print("NOTE 4");
    if (type_tone == 0) {
      display.print("   *");
    }
    display.display();
    analogWrite(LED4, 255);
    playNote(NOTE4);
    
  } 
  delay(100);
}

void playNote(int noteFrequency) {
  int toneDuration = 800;
  int silenceDuration = 0;

  while (analogRead(PHOTO1_PIN) < intensity || analogRead(PHOTO2_PIN) < intensity || analogRead(PHOTO3_PIN) < intensity || analogRead(PHOTO4_PIN) < intensity) {
    
    tone(BUZZER_PIN, noteFrequency, toneDuration);
      
  }
  analogWrite(LED1, 0);
  analogWrite(LED2, 0);
  analogWrite(LED3, 0);
  analogWrite(LED4, 0);
  noTone(BUZZER_PIN);
}


//  if (photo1 >= 500 && photo2 >= 500 && photo3 >= 500 &&
//      photo4 >= 500) {
//    noTone(BUZZER_PIN);
//    lcd_1.clear();
//    lcd_1.print("Vrei sa canti?");
//  }
//  
//    if (photo1 < 500) {
//      playNote(NOTE_C4);
//      lcd_1.clear();
//      lcd_1.print("Photoresistor 1");
//      digitalWrite(LED1, HIGH);
//    } else if (photo2 < 500) {
//      playNote(NOTE_D4);
//      lcd_1.clear();
//      lcd_1.print("Photoresistor 2");
//      digitalWrite(LED2, HIGH);
//    } else if (photo3 < 500) {
//      playNote(NOTE_E4);
//      lcd_1.clear();
//      lcd_1.print("Photoresistor 3");
//      digitalWrite(LED3, HIGH);
//    } else if (photo4 < 500) {
//      playNote(NOTE_F4);
//      lcd_1.clear();
//      lcd_1.print("Photoresistor 4");
//      digitalWrite(LED4, HIGH);
//    } 
